==> abg_utils-4.2.8.tar.gz: lib/src/model/cart.dart <==

import '../../abg_utils.dart';

List<ProductData> cart = [];
ProviderData? cartCurrentProvider;
bool cartAnyTime = true;
var cartSelectTime = DateTime.now();
String cartHint = "";
String cartCouponCode = "";
/*
вся информация в price.dart:
String couponId = "";     // 2746fde7643fgd
String couponCode = "";   // CODE25
String discountType = ""; // "percent" or "fixed"
double discount = 0;      // 12
 */

clearCart(){
  // checkout = [];
  cartCouponCode = "";
  // coupon = null;
  // cachePayment = true;
  cartHint = "";
  cartAnyTime = true;
  cartSelectTime = DateTime.now();
  // paymentMethod = 1;
}

// List<String> checkout = []; // provider id

String? addToCart(ProductData service,
    String stringAlreadyInCart  /// strings.get(210) /// This service already in the cart
    ){
  for (var item in cart)
    if (item.id == service.id)
      return stringAlreadyInCart;
  cart.add(service);
  _cartSave();
}

removeFromCart(ProductData service){
  for (var item in cart)
    if (item.id == service.id){
      cart.remove(item);
      _cartSave();
      return;
    }
}

_cartSave(){
  //userAccountData.cartList = [];
  // for (var item in cart)
  //   userAccountData.cartList.add(item.id);
  // User? user = FirebaseAuth.instance.currentUser;
  // if (user == null)
  //   return "user == null";
  // FirebaseFirestore.instance.collection("listusers").doc(user.uid).set({
  //   "cartList": userAccountData.cartList,
  // }, SetOptions(merge:true));
  // redrawMainWindow();
}

initCart(){
  // lastBookingId = "";
  // if (_init)
  //   return;
  // _init = true;
  // for (var providerId in userAccountData.cartList){
  //   for (var service in product)
  //     if (service.id == providerId) {
  //       bool _found = false;
  //       for (var item in cart )
  //         if (item.id == service.id)
  //           _found = true;
  //       if (!_found)
  //         cart.add(service);
  //     }
  // }
}

_removeCheckoutFromCart(){
  for (var item in userAccountData.cartList)
    for (var service in cart)
      if (service.id == item){
        cart.remove(service);
        break;
      }
  _cartSave();
}

// _allCheckout(){
//   checkout = [];
//   for (var item in cart)
//     if (item.providers.isNotEmpty)
//       checkout.add(item.providers[0]);
// }

checkoutProvider(ProviderData provider){
  cartCurrentProvider = provider;
  // checkout = [];
  // for (var item in cart)
  //   if (item.providers.isNotEmpty)
  //     if (item.providers[0] == provider.id)
  //       checkout.add(item.providers[0]);
}


Future<String?> finish(String paymentMethod) async{
  // for (var item in userAccountData.cartList){
  //   for (var service in cart){
  //     if (service.id == item) {
  //       var ret = await _finish(_mainModel, service, paymentMethod);
  //       if (ret != null)
  //         messageError(parent.context, ret);
  //     }
  //   }
  // }
  // _removeCheckoutFromCart();
  // clearCart();
}

List<String> _cartGetProviders(){
  List<String> ret = [];
  for (var item in cart)
    if (item.providers.isNotEmpty)
      if (!ret.contains(item.providers[0]))
        ret.addAll(item.providers);
  return ret;
}

List<ProviderData> cartGetProvidersData(){
  List<ProviderData> ret = [];
  List<String> list = _cartGetProviders();
  for (var item in list){
    var t = getProviderById(item);
    if (t != null)
      ret.add(t);
  }
  return ret;
}

List<PriceForCardData> cartGetPriceForAllServices(){
  List<PriceForCardData> list = [];

  for (var service in cart){
    if (cartCurrentProvider != null && service.providers.isNotEmpty)
      if (service.providers[0] == cartCurrentProvider!.id){
        if (service.thisIsArticle){
          currentArticle = service;
          var _price = articleGetTotalPrice();
          var name = articleGetFullName();
          list.add(PriceForCardData(_price ?? 0, 0, _price ?? 0, name,
            getPriceString(_price ?? 0),
            getPriceString(0),
            getPriceString(_price ?? 0),
            true
          ));
        }else{
          setDataToCalculate(null, service);
          countProduct = service.count;

          var _price = getCostWidthCount();
          var addons = getAddonsTotal();
          var name = getTextByLocale(service.name, locale) + " " + getTextByLocale(price.name, locale);

          list.add(PriceForCardData(_price, addons, _price+addons, name,
            getPriceString(_price),
            getPriceString(addons),
            getPriceString(_price+addons),
            false
          ));
        }
      }
  }

  return list;
}

PriceTotalForCardData cartGetTotalForAllServices(){
  double coupon = 0;
  double tax = 0;
  double total = 0;
  for (var service in cart){
    if (cartCurrentProvider != null && service.providers.isNotEmpty)
      if (service.providers[0] == cartCurrentProvider!.id){
        if (service.thisIsArticle){
          currentArticle = service;
          var t = articleGetTotalPrice();
          if (t != null) {
            var _tax = service.tax/100*t;
            tax += _tax;
            total += (t+_tax);
          }
        }else{
          setDataToCalculate(null, service);
          countProduct = service.count;
          coupon += getCoupon();
          tax += getTax();
          total += getTotal();
        }
    }
  }
  return PriceTotalForCardData(coupon, tax, total,
    getPriceString(coupon),
    getPriceString(tax),
    getPriceString(total),
  );
}


class PriceForCardData{
  final String name;
  final double priceWithCount;
  final double priceAddons;
  final double subTotal;
  final String priceString;
  final String priceAddonsString;
  final String subTotalString;
  final bool isArticle;
  PriceForCardData(this.priceWithCount, this.priceAddons, this.subTotal, this.name,
      this.priceString, this.priceAddonsString, this.subTotalString, this.isArticle);
}

class PriceTotalForCardData{
  final double discount;
  final double tax;
  final double total;
  final String discountString;
  final String taxString;
  final String totalString;
  PriceTotalForCardData(this.discount, this.tax, this.total,
      this.discountString, this.taxString, this.totalString);
}

==> abg_utils-4.2.9.tar.gz: lib/src/model/cart.dart <==

import '../../abg_utils.dart';

List<ProductData> cart = [];
ProviderData? cartCurrentProvider;
bool cartAnyTime = true;
var cartSelectTime = DateTime.now();
String cartHint = "";
String cartCouponCode = "";
/*
вся информация в price.dart:
String couponId = "";     // 2746fde7643fgd
String couponCode = "";   // CODE25
String discountType = ""; // "percent" or "fixed"
double discount = 0;      // 12
 */

clearCart(){
  // checkout = [];
  cartCouponCode = "";
  // coupon = null;
  // cachePayment = true;
  cartHint = "";
  cartAnyTime = true;
  cartSelectTime = DateTime.now();
  // paymentMethod = 1;
}

// List<String> checkout = []; // provider id

String? addToCart(ProductData service,
    String stringAlreadyInCart  /// strings.get(210) /// This service already in the cart
    ){
  for (var item in cart)
    if (item.id == service.id)
      return stringAlreadyInCart;
  cart.add(service);
  _cartSave();
}

removeFromCart(ProductData service){
  for (var item in cart)
    if (item.id == service.id){
      cart.remove(item);
      _cartSave();
      return;
    }
}

_cartSave(){
  //userAccountData.cartList = [];
  // for (var item in cart)
  //   userAccountData.cartList.add(item.id);
  // User? user = FirebaseAuth.instance.currentUser;
  // if (user == null)
  //   return "user == null";
  // FirebaseFirestore.instance.collection("listusers").doc(user.uid).set({
  //   "cartList": userAccountData.cartList,
  // }, SetOptions(merge:true));
  // redrawMainWindow();
}

initCart(){
  // lastBookingId = "";
  // if (_init)
  //   return;
  // _init = true;
  // for (var providerId in userAccountData.cartList){
  //   for (var service in product)
  //     if (service.id == providerId) {
  //       bool _found = false;
  //       for (var item in cart )
  //         if (item.id == service.id)
  //           _found = true;
  //       if (!_found)
  //         cart.add(service);
  //     }
  // }
}

_removeCheckoutFromCart(){
  for (var item in userAccountData.cartList)
    for (var service in cart)
      if (service.id == item){
        cart.remove(service);
        break;
      }
  _cartSave();
}

// _allCheckout(){
//   checkout = [];
//   for (var item in cart)
//     if (item.providers.isNotEmpty)
//       checkout.add(item.providers[0]);
// }

checkoutProvider(ProviderData provider){
  cartCurrentProvider = provider;
  // checkout = [];
  // for (var item in cart)
  //   if (item.providers.isNotEmpty)
  //     if (item.providers[0] == provider.id)
  //       checkout.add(item.providers[0]);
}


Future<String?> cartFinish(String paymentMethod) async{

}

List<ProductData> cartGetProductsForBooking(){
  List<ProductData> list = [];
  for (var service in cart){
    if (cartCurrentProvider != null && service.providers.isNotEmpty)
      if (service.providers[0] == cartCurrentProvider!.id){
        list.add(service);
      }
  }
  return list;
}

List<String> _cartGetProviders(){
  List<String> ret = [];
  for (var item in cart)
    if (item.providers.isNotEmpty)
      if (!ret.contains(item.providers[0]))
        ret.addAll(item.providers);
  return ret;
}

List<ProviderData> cartGetProvidersData(){
  List<ProviderData> ret = [];
  List<String> list = _cartGetProviders();
  for (var item in list){
    var t = getProviderById(item);
    if (t != null)
      ret.add(t);
  }
  return ret;
}

List<PriceForCardData> cartGetPriceForAllServices(){
  List<PriceForCardData> list = [];

  for (var service in cart){
    if (cartCurrentProvider != null && service.providers.isNotEmpty)
      if (service.providers[0] == cartCurrentProvider!.id){
        if (service.thisIsArticle){
          currentArticle = service;
          double _price = 0;
          var t = articleGetTotalPrice();
          if (t != null)
            _price = t * service.count;
          var name = articleGetFullName();
          list.add(PriceForCardData(_price, 0, _price, name,
            getPriceString(_price),
            getPriceString(0),
            getPriceString(_price),
            true
          ));
        }else{
          setDataToCalculate(null, service);
          countProduct = service.count;

          var _price = getCostWidthCount();
          var addons = getAddonsTotal();
          var name = getTextByLocale(service.name, locale) + " " + getTextByLocale(price.name, locale);

          list.add(PriceForCardData(_price, addons, _price+addons, name,
            getPriceString(_price),
            getPriceString(addons),
            getPriceString(_price+addons),
            false
          ));
        }
      }
  }

  return list;
}

PriceTotalForCardData cartGetTotalForAllServices(){
  double coupon = 0;
  double tax = 0;
  double total = 0;
  for (var service in cart){
    if (cartCurrentProvider != null && service.providers.isNotEmpty)
      if (service.providers[0] == cartCurrentProvider!.id){
        if (service.thisIsArticle){
          currentArticle = service;
          var t = articleGetTotalPrice();
          if (t != null) {
            t *= service.count;
            var _tax = service.tax/100*t;
            tax += _tax;
            total += (t+_tax);
          }
        }else{
          setDataToCalculate(null, service);
          countProduct = service.count;
          coupon += getCoupon();
          tax += getTax();
          total += getTotal();
        }
    }
  }
  return PriceTotalForCardData(coupon, tax, total,
    getPriceString(coupon),
    getPriceString(tax),
    getPriceString(total),
  );
}


class PriceForCardData{
  final String name;
  final double priceWithCount;
  final double priceAddons;
  final double subTotal;
  final String priceString;
  final String priceAddonsString;
  final String subTotalString;
  final bool isArticle;
  PriceForCardData(this.priceWithCount, this.priceAddons, this.subTotal, this.name,
      this.priceString, this.priceAddonsString, this.subTotalString, this.isArticle);
}

class PriceTotalForCardData{
  final double discount;
  final double tax;
  final double total;
  final String discountString;
  final String taxString;
  final String totalString;
  PriceTotalForCardData(this.discount, this.tax, this.total,
      this.discountString, this.taxString, this.totalString);
}
